/*
 * ptyInit.c--
 *
 * Initialization for the pty command.
 *
 * Copyright (C) 1997
 *	Dr. Duncan McLennan Barclay.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY DUNCAN BARCLAY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL DUNCAN BARCLAY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ptyInit.c,v 1.6 1997/02/21 22:57:20 dmlb Exp $
 *
 */

#include "tcl.h"
#include "pty.h"

#include <sys/errno.h>

static uid_t ptySavedUid;
static gid_t ptySavedGid;


/*
 *----------------------------------------------------------------------
 *
 * Pty_Init--
 *
 *   Initialization
 *
 *----------------------------------------------------------------------
 */
int
Pty_Init (interp)
  Tcl_Interp *interp;
{

  if (Tcl_PkgRequire (interp, "Tcl", TCL_VERSION, 0) == NULL) {
    return TCL_ERROR;
  }
  if (Tcl_PkgProvide (interp, "Pty", PTY_VERSION) != TCL_OK) {
    return TCL_ERROR;
  }

  /*
   * Deal with suid operation
   */
  ptySavePrivs ();
  if (ptyReleasePrivs () < 0) {
    Tcl_AppendResult (interp, "couldn't release set uid/gid privilages",
                      (char *)NULL);
    return TCL_ERROR;
  }

  /*
   * pty command
   */
  Tcl_CreateCommand (interp, "pty", ptyCmd,
                     (ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);
}

/*
 *----------------------------------------------------------------------
 *
 * ptySavePrivs, ptyGetPrivs, ptyReleasePrivs--
 *
 *   Try and make this a bit more secure
 *
 *----------------------------------------------------------------------
 */
void
ptySavePrivs ()
{
  ptySavedUid = geteuid ();
  ptySavedGid = getegid ();
}

int
ptyGetPrivs ()
{
  int errorCode = 0;

  if (seteuid (ptySavedUid) < 0) {
    Tcl_SetErrno (EPERM);
    errorCode = -1;
  }
  if (setegid (ptySavedGid) < 0) {
    Tcl_SetErrno (EPERM);
    errorCode = -1;
  }

  return errorCode;
}

int
ptyReleasePrivs ()
{
  int errorCode = 0;

  if (seteuid (getuid ()) < 0) {
    Tcl_SetErrno (EPERM);
    errorCode = -1;
  }
  if (setegid (getgid ()) < 0) {
    Tcl_SetErrno (EPERM);
    errorCode = -1;
  }

  return errorCode;
}
