#
# This script implements a simple window to a shell for
# Tk applications.
#

#
# tksh
#
#   Create text window and scrollbar, and set up bindings for text
#   window to execute commands
#
proc tksh {window shell} {

  global tksh_info

  set tksh_info($window,command) ""

  set tksh_info($window,pty) [pty \
    -echo \
    -term xterm \
    -rows 24 -columns 80 \
    -login $shell]
  fconfigure $tksh_info($window,pty) \
    -blocking 0 -buffering line
#  puts $tksh_info($window,pty) "stty -echo"
  fileevent $tksh_info($window,pty) readable [list tksh_readPty $window]

  text $window.t \
    -relief groove -bd 2 -yscrollcommand "$window.s set" -setgrid true
  bindtags $window.t tksh_bindings

  scrollbar $window.s -relief groove -command "$window.t yview"

  pack $window.s -side left -fill both
  pack $window.t -side right -expand true

}

#
# Bindings for tksh
#
bind tksh_bindings <Configure> {
  set g [wm geometry [winfo toplevel %W]]
  set c [string range $g 0 [expr [string first x $g]-1]]
  set r [string range $g \
    [expr [string first x $g]+1] [expr [string first + $g]-1]]
  %W configure -width $c -height $r
}

bind tksh_bindings <Return> {
  tksh_insertKeyPress [winfo parent %W] "\n"
  tksh_invoke [winfo parent %W]
}
bind tksh_bindings <KeyPress> {
  tksh_insertKeyPress [winfo parent %W] %A
}
bind tksh_bindings <BackSpace> {
  tksh_backspace [winfo parent %W]
}
bind tksh_bindings <Delete> {
  tksh_backspace [winfo parent %W]
}
bind tksh_bindings <Control-h> {
  tksh_backspace [winfo parent %W]
}

bind tksh_bindings <1> {
  tksh_buttonPress1 [winfo parent %W]
}

#XXX are these right?
#  bind $window <FocusIn> {focus $window.t}
#  bind $window <FocusOut> {focus none}
#  bind $window.t <Enter> {focus $window.t}
#  bind $window.s <Enter> {focus $window.t}
#  bind $window.t <1> {
#      set tk_priv(selectMode) char
#      %W mark set anchor @%x,%y
#      if {[lindex [%W config -state] 4] == "normal"} {focus %W}
#  }
#  bind $window.t <Double-1> {
#      set tk_priv(selectMode) word
#      tk_textSelectTo %W @%x,%y
#  }
#  bind $window.t <Triple-1> {
#      set tk_priv(selectMode) line
#      tk_textSelectTo %W @%x,%y
#  }
#  bind $window.t <2> {
#    .t insert insert [selection get]
#    .t yview -pickplace insert
#    if [string match *.0 [.t index insert]] {
#	tksh_invoke
#    }
#  }
#  bind $window.t <Control-c> {
#      .t delete {promptEnd + 1 char} insert
#      .t yview -pickplace insert
#  }
#  bind $window.t <Control-d> {
#    after 0 {destroy .}
#  }

#
# tksh_insertKeyPress
#
#   Processes key presses in the text window,
#   updates the current command line.
#
proc tksh_insertKeyPress {window string} {

  global tksh_info

  if {($string == "") || ([$window.t cget -state] == "disabled")} {
    return
  }
  $window.t insert insert $string
  append tksh_info($window,command) $string
  $window.t see insert
}

#
# tksh_backspace
#
#   Process a delete and updates the current command line.
# 
proc tksh_backspace {window} {

  global tksh_info

  if {[$window.t compare insert != 1.0]} {
    $window.t delete insert-1c
    $window.t see insert
  }
  set tksh_info($window,command) [
    string range $tksh_info($window,command) \
      0 [expr [string length $tksh_info($window,command)]-2] \
  ]
}

#
# tksh_buttonPress1
#
#   Process a button 1 press in the shell window,
#   sets the focus etc.
#
proc tksh_buttonPress1 {window} {

  if {[$window.t cget -state] == "normal"} {
    focus $window.t
  }
}

#
# tksh_invoke
#
#   The procedure below executes a command (it takes everything typed by
#   the user and then passes it to the shell)
#
proc tksh_invoke {window} {

    global tksh_info

    puts -nonewline $tksh_info($window,pty) $tksh_info($window,command)
    set tksh_info($window,command) ""
 
    $window.t see insert
}

#
# tksh_readPty
#
#   reads data from the pty when it is ready and writes it to the text
#   widget
# 
proc tksh_readPty {window} {

  global tksh_info errorCode errorInfo

  set t [read $tksh_info($window,pty)]
  if {![eof $tksh_info($window,pty)]} {
    $window.t insert insert $t
    $window.t see insert
  } else {
    fconfigure $tksh_info($window,pty) -blocking 1
    catch {close $tksh_info($window,pty)}
    bindtags $window.t {}
    destroy $window.t
    destroy $window.s
    foreach e [array names tksh_info "$window*"] {
      unset tksh_info($e)
    }
  }
}
